function [onset,offset,classNames] =  event_detection(filename,iter,S,sz,su)
% e.g. [onset,offset,classNames] = event_detection('office_snr0_high.wav',30,60,1.1,1.4);
%
% Polyphonic Sound Event Tracking using Linear Dynamical Systems
%
% SED system trained on DCASE 2013 - OS development dataset
% Optimised for frame-based F-measure
%
% Inputs:
%  filename filename for .wav file
%  iter     number of iterations (e.g. 30)
%  S        number of exemplars per event class (set to 60 from the training dataset)
%  sz       sparsity parameter for event activation (e.g. 0.9-1.5)
%  su       sparsity parameter for exemplar contribution (e.g. 0.9-1.5)
%
% Outputs:
%  onset:      vector of onset times for each detected event
%  offset:     vector of offset times for each detected event
%  classNames: cell vector of event classes for each detected event 
%
% Emmanouil Benetos 2017


% Load spectral templates and initialize
load('shiftedW.mat');
W = permute(shiftedW,[4 5 2 1 3]);
W = permute(W,[1 3 2 4 5]);
W = W(:,1:S,:,:,:);


% Compute ERB
X = computeERB(filename)'; % 23.22ms step


% Emphasize high freqs and remove low frequency bins
f = X(:,21:end)';
f = f .* repmat( linspace( 1, 15, size( f, 1))', 1, size( f, 2));
X =f';


% Remove background noise from input
p1 = prctile(sum(X'),10);
p2 = prctile(sum(X'),30);
silentFrames = find(sum(X')>p1 & sum(X')<p2);
silentTemplate = mean(X(silentFrames,:));
noiseLevel = repmat(silentTemplate,[size(X,1) 1]);
Y = X - noiseLevel;
Y = max(Y,0);


% Perform efficient 5-D PLCA with LDS prior
[w,h,z,u,g,xa] = plca_5d_fast_lds( Y', 16, S, 1, 3, iter, 1.0, sz, su, 1.0, W, [], [], [], [], 0);


% Postprocessing
eventRoll = z';

[B,IX] =sort(eventRoll,2,'descend');  % Max polyphony
tempEventRoll = zeros(size(eventRoll,1),16);
for j=1:size(eventRoll,1) for k=1:5 tempEventRoll(j,IX(j,k)) = B(j,k); end; end;
eventRoll = tempEventRoll;

expandedEventRoll = zeros(round(2.322*size(eventRoll,1)),16); % interpolate to 10ms step
for j=1:round(2.322*size(eventRoll,1))
    expandedEventRoll(j,:) = eventRoll(floor((j-1)/2.322)+1,:);
end;


% Thresholding (settings from OS dev)
th = [10.6 40.0 39.6 38.6 28.0 36.0 6.8 17.6 20.2 3.8 38.4 39.8 5.6 16.2 26.8 39.20];
path = [];
for k=1:16 
    path(:,k) = (expandedEventRoll(:,k) > th(k)); 
end;


% Minimum duration pruning (60ms) & convert to list of detected events
[onset,offset,classNames] = convertEventRolltoEventList(path,0.06,0.06);


% Write output to .txt file
filename = strrep(filename,'.wav','');
fid=fopen([filename '_sed.txt'],'w');
for i=1:length(onset)
    fprintf(fid,'%.2f\t%.2f\t%s\n',onset(i),offset(i),classNames{i});
end;
fclose(fid);
fprintf('%s','done');
fprintf('\n');
